@@ -11,9 +11,9 @@ from typing import overload
 import operator
 from collections.abc import Iterator
 from datetime import datetime
-from datetime import timezone
+from datetime import UTC
 
-import cairo
+from gi.repository import Gdk
 from gi.repository import GLib
 from nbxmpp.const import Affiliation
 from nbxmpp.const import Chatstate
@@ -28,8 +28,8 @@ from nbxmpp.structs import MucSubject
 from nbxmpp.structs import TuneData
 
 from gajim.common import app
+from gajim.common import modules
 from gajim.common import types
-from gajim.common.client_modules import ClientModules
 from gajim.common.const import PresenceShowExt
 from gajim.common.const import SimpleClientState
 from gajim.common.helpers import Observable
@@ -146,11 +146,17 @@ class Contacts(BaseModule):
 
         contact = self._contacts.get(jid)
         if contact is not None:
-            if not isinstance(contact, GroupchatContact):
-                raise ValueError(f'Trying to add GroupchatContact {jid}, '
-                                 f'but contact already exists as {contact} '
-                                 f'(in roster: {contact.is_in_roster})')
-            return contact
+            if isinstance(contact, GroupchatContact):
+                return contact
+
+            self._log.error(f'Trying to add GroupchatContact {jid}, '
+                            f'but contact already exists as {contact} '
+                            f'(in roster: {contact.is_in_roster})')
+            # Most likely a groupchat which has been added to the roster.
+            # Remove jid from roster (if we are online) and delete contact.
+            if self._client.state.is_available:
+                self._client.get_module('Roster').delete_item(jid)
+            self._contacts.pop(jid)
 
         contact = GroupchatContact(self._log, jid, self._account)
 
@@ -247,7 +253,7 @@ class Contacts(BaseModule):
             contact.force_chatstate_update()
 
 
-class CommonContact(Observable, ClientModules):
+class CommonContact(Observable):
     def __init__(self,
                  logger: LogAdapter,
                  jid: JID,
@@ -255,7 +261,6 @@ class CommonContact(Observable, ClientModules):
                  ) -> None:
 
         Observable.__init__(self, logger)
-        ClientModules.__init__(self, account)
         self._jid = jid
         self._account = account
         self._gateway_type: str | None = None
@@ -278,6 +283,9 @@ class CommonContact(Observable, ClientModules):
     def account(self) -> str:
         return self._account
 
+    def get_module(self, name: modules.ModulesLiteralT) -> modules.ModulesT:
+        return modules.get_module(self._account, name)
+
     def _on_signal(
         self,
         _contact: (BareContact |
@@ -339,7 +347,7 @@ class CommonContact(Observable, ClientModules):
             return False
 
         until = datetime.fromisoformat(mute_until)
-        is_muted = until > datetime.now(timezone.utc)
+        is_muted = until > datetime.now(UTC)
         if not is_muted:
             # Reset the setting to default
             GLib.idle_add(self.settings.set, 'mute_until', None)
@@ -541,7 +549,7 @@ class BareContact(CommonContact):
         return app.storage.cache.get_contact(
             self._account, self._jid, 'avatar')
 
-    def set_avatar_sha(self, sha: str) -> None:
+    def set_avatar_sha(self, sha: str | None) -> None:
         app.storage.cache.set_contact(self._account, self._jid, 'avatar', sha)
 
     def get_avatar(self,
@@ -549,13 +557,14 @@ class BareContact(CommonContact):
                    scale: int,
                    add_show: bool = True,
                    default: bool = False,
-                   style: str = 'circle'):
+                   style: str = 'circle'
+                   ) -> Gdk.Texture:
 
         show = self.show.value if add_show else None
 
         transport_icon = self._get_transport_icon_name()
 
-        return app.app.avatar_storage.get_surface(
+        return app.app.avatar_storage.get_texture(
             self,
             size,
             scale,
@@ -571,7 +580,7 @@ class BareContact(CommonContact):
                 contact.update_presence(presence_data, notify=False)
         self.notify('presence-update')
 
-    def update_avatar(self, sha: str) -> None:
+    def update_avatar(self, sha: str | None) -> None:
         if self._avatar_sha == sha:
             return
 
@@ -640,7 +649,7 @@ class BareContact(CommonContact):
         return True
 
     @property
-    def type_string(self) -> str:
+    def type_string(self) -> Literal['chat']:
         return 'chat'
 
     @property
@@ -815,6 +824,12 @@ class GroupchatContact(CommonContact):
             if contact.is_available:
                 yield contact
 
+    def get_occupant(self, occupant_id: str) -> GroupchatParticipant | None:
+        for contact in self._resources.values():
+            if contact.occupant_id == occupant_id:
+                return contact
+        return None
+
     @property
     def name(self) -> str:
         client = app.get_client(self._account)
@@ -836,13 +851,13 @@ class GroupchatContact(CommonContact):
     def avatar_sha(self) -> str | None:
         return app.storage.cache.get_muc(self._account, self._jid, 'avatar')
 
-    def set_avatar_sha(self, sha: str) -> None:
+    def set_avatar_sha(self, sha: str | None) -> None:
         app.storage.cache.set_muc(self._account, self._jid, 'avatar', sha)
 
-    def get_avatar(self, size: int, scale: int) -> cairo.ImageSurface:
+    def get_avatar(self, size: int, scale: int) -> Gdk.Texture:
         transport_icon = self._get_transport_icon_name()
 
-        return app.app.avatar_storage.get_muc_surface(
+        return app.app.avatar_storage.get_muc_texture(
             self._account,
             self._jid,
             size,
@@ -902,7 +917,7 @@ class GroupchatContact(CommonContact):
         muc_data = self.get_module('MUC').get_muc_data(self._jid)
         if muc_data is None:
             return False
-        return muc_data.state.is_joining
+        return muc_data.state.is_joining or muc_data.state.is_before_joining
 
     @property
     def is_not_joined(self) -> bool:
@@ -917,7 +932,7 @@ class GroupchatContact(CommonContact):
 
     def get_user_nicknames(self) -> list[str]:
         client = app.get_client(self._account)
-        return client.get_module('MUC').get_joined_users(str(self._jid))
+        return client.get_module('MUC').get_joined_users(self._jid)
 
     def get_disco(self, max_age: int = 0) -> DiscoInfo | None:
         return app.storage.cache.get_last_disco_info(self.jid, max_age=max_age)
@@ -928,7 +943,7 @@ class GroupchatContact(CommonContact):
         return all_ or room
 
     @property
-    def type_string(self) -> str:
+    def type_string(self) -> Literal['groupchat']:
         return 'groupchat'
 
     def has_composing_participants(self) -> bool:
@@ -1047,10 +1062,10 @@ class GroupchatParticipant(CommonContact):
                    scale: int,
                    add_show: bool = True,
                    style: str = 'circle'
-                   ) -> cairo.ImageSurface:
+                   ) -> Gdk.Texture:
 
         show = self.show.value if add_show else None
-        return app.app.avatar_storage.get_surface(
+        return app.app.avatar_storage.get_texture(
             self, size, scale, show, style=style)
 
     def update_presence(self, presence: MUCPresenceData) -> None:
@@ -1061,7 +1076,7 @@ class GroupchatParticipant(CommonContact):
         self.notify('user-avatar-update')
 
     @property
-    def type_string(self) -> str:
+    def type_string(self) -> Literal['pm']:
         return 'pm'
 
     @property
@@ -1078,17 +1093,14 @@ class GroupchatParticipant(CommonContact):
     def hats(self) -> HatData | None:
         return self._presence.hats
 
+    @property
+    def is_blocked(self) -> bool:
+        client = app.get_client(self._account)
+        return client.get_module('MucBlocking').is_blocked(
+            self.room.jid, self.occupant_id)
 
-def can_add_to_roster(
-    contact: BareContact | GroupchatContact | GroupchatParticipant
-) -> bool:
-
-    if isinstance(contact, GroupchatContact):
-        return False
-
-    if isinstance(contact, GroupchatParticipant):
-        return contact.real_jid is not None
 
+def can_add_to_roster(contact: BareContact) -> bool:
     if contact.is_self:
         return False
     return not contact.is_in_roster
