@@ -34,12 +34,17 @@ from gajim.common.helpers import load_file_async
 from gajim.common.helpers import write_file_async
 from gajim.common.i18n import _
 from gajim.common.multiprocess.thumbnail import create_thumbnail
+from gajim.common.multiprocess.video_thumbnail import (
+    extract_video_thumbnail_and_properties,
+)
 from gajim.common.storage.archive import models as mod
 from gajim.common.util.http import create_http_request
 from gajim.common.util.preview import aes_decrypt
 from gajim.common.util.preview import filename_from_uri
 from gajim.common.util.preview import get_image_paths
+from gajim.common.util.preview import get_previewable_image_mime_types
 from gajim.common.util.preview import get_previewable_mime_types
+from gajim.common.util.preview import get_previewable_video_mime_types
 from gajim.common.util.preview import guess_mime_type
 from gajim.common.util.preview import parse_fragment
 from gajim.common.util.preview import split_geo_uri
@@ -49,11 +54,14 @@ log = logging.getLogger('gajim.c.preview')
 
 IRI_RX = re.compile(regex.IRI)
 
+PREVIEWABLE_IMAGE_MIME_TYPES = get_previewable_image_mime_types()
+PREVIEWABLE_VIDEO_MIME_TYPES = get_previewable_video_mime_types()
 PREVIEWABLE_MIME_TYPES = get_previewable_mime_types()
 mime_types = set(MIME_TYPES)
 # Merge both: if it’s a previewable image, it should be allowed
 ALLOWED_MIME_TYPES = mime_types.union(PREVIEWABLE_MIME_TYPES)
 
+
 AudioSampleT = list[tuple[float, float]]
 
 
@@ -128,6 +136,16 @@ class Preview:
         is_allowed = bool(self.mime_type in ALLOWED_MIME_TYPES)
         return is_allowed and self.mime_type.startswith('audio/')
 
+    @property
+    def is_image(self) -> bool:
+        is_allowed = bool(self.mime_type in ALLOWED_MIME_TYPES)
+        return is_allowed and self.mime_type.startswith('image/')
+
+    @property
+    def is_video(self) -> bool:
+        is_allowed = bool(self.mime_type in ALLOWED_MIME_TYPES)
+        return is_allowed and self.mime_type.startswith('video/')
+
     @property
     def uri(self) -> str:
         return self._uri
@@ -350,9 +368,13 @@ class PreviewManager:
 
         elif not preview.thumb_exists:
             assert preview.orig_path is not None
-            load_file_async(preview.orig_path,
-                            self._on_orig_load_finished,
-                            preview)
+            preview.mime_type = guess_mime_type(preview.orig_path)
+            preview.file_size = os.path.getsize(preview.orig_path)
+            preview.update_widget()
+            if preview.is_image:
+                self._create_thumbnail(preview)
+            elif preview.is_video:
+                self._create_video_thumbnail(preview)
 
         else:
             assert preview.thumb_path is not None
@@ -360,40 +382,89 @@ class PreviewManager:
                             self._on_thumb_load_finished,
                             preview)
 
-    def create_thumbnail(self, preview: Preview, data: bytes) -> None:
+    @staticmethod
+    def _on_thumb_load_finished(data: bytes | None,
+                                error: GLib.Error | None,
+                                preview: Preview) -> None:
+
+        if preview.thumb_path is None or preview.orig_path is None:
+            return
+
+        if data is None:
+            log.error('%s: %s', preview.thumb_path.name, error)
+            return
+
+        preview.thumbnail = data
+        # Thumbnails are stored always as PNG, we don’t know the
+        # mime-type of the original picture
+        assert preview.orig_path is not None
+        preview.mime_type = guess_mime_type(preview.orig_path)
+        preview.file_size = os.path.getsize(preview.orig_path)
+
+        preview.update_widget(data=data)
+
+    def _create_thumbnail(self, preview: Preview) -> None:
+        assert preview.thumb_path is not None
+        assert preview.orig_path is not None
         try:
             future = app.process_pool.submit(
                 create_thumbnail,
-                data,
+                preview.orig_path,
+                preview.thumb_path,
                 preview.size,
                 preview.mime_type,
             )
-            future.add_done_callback(partial(self._write_thumbnail, preview))
+            future.add_done_callback(
+                partial(GLib.idle_add, self._create_thumbnail_finished, preview)
+            )
         except Exception as error:
             preview.info_message = _('Creating thumbnail failed')
             preview.update_widget()
             log.warning('Creating thumbnail failed for: %s %s',
                         preview.orig_path, error)
 
-    def _write_thumbnail(self, preview: Preview, future: Future[bytes | None]) -> None:
-        try:
-            result = future.result()
-            if result is None:
-                raise ValueError("result is None")
-        except Exception as error:
-            preview.info_message = _('Creating thumbnail failed')
-            preview.update_widget()
-            log.warning('Creating thumbnail failed for: %s %s',
-                        preview.orig_path, error)
+    def _create_video_thumbnail(self, preview: Preview) -> None:
+        if preview.thumb_path is None:
+            log.warning("Creating thumbnail failed, thumbnail path is None")
             return
 
-        preview.thumbnail = result
+        assert preview.orig_path is not None
 
-        assert preview.thumb_path is not None
-        write_file_async(preview.thumb_path,
-                         preview.thumbnail,
-                         self._on_thumb_write_finished,
-                         preview)
+        try:
+            future = app.process_pool.submit(
+                extract_video_thumbnail_and_properties,
+                preview.orig_path,
+                preview.thumb_path,
+                preview.size
+            )
+            future.add_done_callback(
+                partial(GLib.idle_add,
+                        self._create_thumbnail_finished,
+                        preview)
+            )
+        except Exception as error:
+            preview.info_message = _("Creating thumbnail failed")
+            preview.update_widget()
+            log.warning("Creating thumbnail failed for: %s %s",
+                        preview.orig_path, error)
+
+    def _create_thumbnail_finished(
+        self,
+        preview: Preview,
+        future: Future[tuple[bytes, dict[str, Any]]]
+    ) -> bool:
+        try:
+            thumbnail_bytes, _metadata = future.result()
+        except Exception as error:
+            preview.info_message = _('Creating thumbnail failed')
+            log.exception('Creating thumbnail failed for: %s %s',
+                          preview.orig_path, error)
+        else:
+            preview.thumbnail = thumbnail_bytes
+
+        preview.update_widget(data=preview.thumbnail)
+
+        return GLib.SOURCE_REMOVE
 
     def _process_web_uri(self,
                          uri: str,
@@ -417,45 +488,6 @@ class PreviewManager:
                        from_us,
                        context=context)
 
-    def _on_orig_load_finished(self,
-                               data: bytes | None,
-                               error: GLib.Error | None,
-                               preview: Preview) -> None:
-        if preview.thumb_path is None or preview.orig_path is None:
-            return
-
-        if data is None:
-            log.error('%s: %s', preview.orig_path.name, error)
-            return
-
-        preview.mime_type = guess_mime_type(preview.orig_path, data)
-        preview.file_size = os.path.getsize(preview.orig_path)
-
-        if preview.is_previewable:
-            self.create_thumbnail(preview, data)
-        else:
-            preview.update_widget()
-
-    @staticmethod
-    def _on_thumb_load_finished(data: bytes | None,
-                                error: GLib.Error | None,
-                                preview: Preview) -> None:
-
-        if preview.thumb_path is None or preview.orig_path is None:
-            return
-
-        if data is None:
-            log.error('%s: %s', preview.thumb_path.name, error)
-            return
-
-        preview.thumbnail = data
-        # Thumbnails are stored always as PNG, we don’t know the
-        # mime-type of the original picture
-        preview.mime_type = guess_mime_type(preview.orig_path)
-        preview.file_size = os.path.getsize(preview.orig_path)
-
-        preview.update_widget(data=data)
-
     def download_content(self,
                          preview: Preview,
                          force: bool = False
@@ -481,6 +513,11 @@ class PreviewManager:
 
         request.send('GET', preview.request_uri, callback=self._on_finished)
 
+    def cancel_download(self, preview: Preview) -> None:
+        assert preview.request is not None
+        preview.request.cancel()
+        preview.download_in_progress = False
+
     def _accept_certificate(self,
                             request: HTTPRequest,
                             _certificate: Gio.TlsCertificate,
@@ -572,63 +609,38 @@ class PreviewManager:
                     preview.update_widget()
                     return
 
-        if preview.mime_type == 'application/octet-stream':
-            if preview.orig_path is not None:
-                preview.mime_type = guess_mime_type(preview.orig_path, data)
-
         assert preview.orig_path is not None
+        preview.mime_type = guess_mime_type(preview.orig_path, data)
+
         write_file_async(preview.orig_path,
                          data,
                          self._on_orig_write_finished,
                          preview)
 
-        if not app.settings.get('enable_file_preview'):
-            preview.update_widget()
-            return
-
-        if preview.is_previewable:
-            self.create_thumbnail(preview, data)
-
-    @staticmethod
-    def _on_orig_write_finished(_result: bool,
+    def _on_orig_write_finished(self,
+                                _result: bool,
                                 error: GLib.Error | None,
                                 preview: Preview) -> None:
-        if preview.orig_path is None:
-            return
 
+        assert preview.orig_path is not None
         if error is not None:
             log.error('%s: %s', preview.orig_path.name, error)
             return
 
         log.info('File stored: %s', preview.orig_path.name)
         preview.file_size = os.path.getsize(preview.orig_path)
-        if not preview.is_previewable:
-            # Don’t update preview if thumb is already displayed,
-            # but update preview for audio files
+
+        if not app.settings.get('enable_file_preview'):
             preview.update_widget()
-
-    @staticmethod
-    def _on_thumb_write_finished(_result: bool,
-                                 error: GLib.Error | None,
-                                 preview: Preview) -> None:
-        if preview.thumb_path is None:
             return
 
-        if error is not None:
-            log.error('%s: %s', preview.thumb_path.name, error)
-            if not preview.thumb_exists:
-                # Generating a preview can fail if the file already exists
-                # Only abort if thumbnail has not been stored in preview
-                return
-
-        log.info('Thumbnail stored: %s ', preview.thumb_path.name)
-
-        if preview.thumbnail is None:
+        if not preview.is_previewable:
             return
 
-        preview.update_widget(data=preview.thumbnail)
+        if preview.is_image:
+            self._create_thumbnail(preview)
 
-    def cancel_download(self, preview: Preview) -> None:
-        assert preview.request is not None
-        preview.request.cancel()
-        preview.download_in_progress = False
+        elif preview.is_video:
+            self._create_video_thumbnail(preview)
+
+        preview.update_widget()
