@@ -94,6 +94,7 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
     _header_bar: Adw.HeaderBar = Gtk.Template.Child()
     _app_side_bar: AppSideBar = Gtk.Template.Child()
     _main_stack: MainStack = Gtk.Template.Child()
+    _toast_overlay: Adw.ToastOverlay = Gtk.Template.Child()
 
     def __init__(self) -> None:
         app.window = self
@@ -204,6 +205,9 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
 
         return self._emoji_chooser
 
+    def show_toast(self, toast: Adw.Toast) -> None:
+        self._toast_overlay.add_toast(toast)
+
     def show(self) -> None:
         self.present()
 
@@ -500,28 +504,33 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
 
         control = self.get_control()
         if control.has_active_chat():
+            contact = control.get_contact()
+            assert contact is not None
+
             if action_name == "change-nickname":
                 app.window.activate_action("win.muc-change-nickname", None)
                 return None
 
             if action_name == "change-subject":
-                open_window("GroupchatDetails", contact=control.contact, page="manage")
+                open_window("GroupchatDetails", contact=contact, page="manage")
                 return None
 
             if action_name == "escape":
                 if app.settings.get("escape_key_closes"):
-                    self._chat_page.remove_chat(
-                        control.contact.account, control.contact.jid
-                    )
-                    return None
+                    self._chat_page.remove_chat(contact.account, contact.jid)
+                else:
+                    workspace_id = self.get_active_workspace()
+                    if workspace_id is not None:
+                        app.window.activate_workspace(workspace_id)
 
-            elif action_name == "close-chat":
-                self._chat_page.remove_chat(
-                    control.contact.account, control.contact.jid
-                )
                 return None
 
-        if action_name == "escape" and app.settings.get("escape_key_closes"):
+            elif action_name == "close-chat":
+                self._chat_page.remove_chat(contact.account, contact.jid)
+                return None
+
+        if action_name in ("escape", "close-chat"):
+            # Attempt to close Gajim window if no chat has been selected
             self.close()
 
         elif action_name == "restore-chat":
@@ -923,11 +932,13 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
             return
 
         if self.is_active():
-            client = app.get_client(control.contact.account)
+            contact = control.get_contact()
+            assert contact is not None
+            client = app.get_client(contact.account)
             chat_stack = self._chat_page.get_chat_stack()
             msg_action_box = chat_stack.get_message_action_box()
             client.get_module("Chatstate").set_mouse_activity(
-                control.contact, msg_action_box.msg_textview.has_text
+                contact, msg_action_box.msg_textview.has_text
             )
 
     def _on_chat_selected(self, *args: Any) -> None:
@@ -1282,8 +1293,11 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
         if not control.has_active_chat():
             return
 
+        contact = control.get_contact()
+        assert contact is not None
+
         if control.view_is_at_bottom():
-            self.mark_as_read(control.contact.account, control.contact.jid)
+            self.mark_as_read(contact.account, contact.jid)
 
     def get_preferred_ft_method(self, contact: types.ChatContactT) -> str | None:
 
@@ -1470,6 +1484,8 @@ class MainWindow(Adw.ApplicationWindow, EventHelper):
             self.add_chat(event.account, event.jid, "chat")
 
     def quit(self) -> None:
+        self.show_toast(Adw.Toast(title=_("Gajim is quitting…"), timeout=0))
+
         if self.is_visible():
             window_width, window_height = self.get_width(), self.get_height()
             app.settings.set("mainwin_width", window_width)
